'''Post Event ER

This module defines a function called post_event which creates a report
in Earthranger that shows up on the map and returns the id that event.
'''
import json
from datetime import datetime, timedelta
from earthranger_utils.get_cam_location import cam_location

import requests

def post_event(label, cam_name, token, authorization):
    '''Post Event

    This function takes in the camera name and label of an image and returns
    the unique id of an event created in Earthranger for the specific image to
    be added. It calls cam_location() in order to make the event appear in the 
    correc goegraphical location on the Earthranger instance.

    Args:
    label: a string value of the animal that the image was classified as
    cam_name: a string of the specific name of the camera that the image came from
        as it also is in Earthranger
    token: unique token for ER to authenticate http request, defined in config yml
    authorization: the other auth token for ER as defined in config yml, this was
        retrieved from the interactive api on ER
        https://<YOUR INSTANCE>.pamdas.org/api/v1.0/docs/interactive/
    label: the name of the animal identified to be sent, used as title of jpeg

    Returns: the unique id of the event that was posted to be used to attach
        an image to, event must be created before image is added
    '''
    URL = 'https://sagebrush.pamdas.org/api/v1.0/activity/events/'

    Headers = {
      'X-CSRFToken': token,
      'Authorization': authorization
    }
   
    

    cam, subject_id = cam_location(cam_name, token, authorization)
    lat = cam[1]
    longi = cam[0]

    event_data = {
        "event_type": "cougarvision_detection",
        "priority": 100,
        "state": "active",
        "location": {
            "latitude": lat,
            "longitude": longi
        },
        "event_details": {
            "cam_name": cam_name,
            "animal_label": label
        },
        "event_category": "monitoring",
        "related_subjects": [
                {
                    "content_type": "observations.subject",
                    "id": subject_id
                }
        ]
}
    new_event = requests.post(URL, headers=Headers, json=event_data)
    response_json = new_event.json()

    return response_json['data']['id']



'''Get Cam Location

This script defines a function called cam_location where an http request is performed
to retrieve the lat, long, and unique id for the camera trap that the current classified
image from strikeforce has in earthranger so that the event may be linked to the specific
camera.
'''

import requests

def cam_location(cam_name, token, authorization):
    '''Cam Location

    This function takes in the name of the camera in earthranger and returns
    the lat and longs of that camera as well as earthrangers unique identifyer
    for it.

    Args:
    cam_name: the name (B001, B002...) of the camera as it is in earthranger 'str'
    token: the token for api calls in earthranger 'str'
    authorization: another token for ER api calls as specified in config yml 'str'

    Return: lat and longs of the camera location as a list and the unique id of the camera
        in earthranger as a 'str'
    '''
    
    url = 'https://sagebrush.pamdas.org/api/v1.0/subjects/?name=' + cam_name
    headers = {
        'X-CSRFToken': token,
        'Authorization': authorization,
        'Accept': 'application/json'
    }
    response = requests.get(url, headers=headers)
    response_json = response.json()

    cam_locations = response_json['data'][0]['last_position']['geometry']['coordinates']
    subject_id = response_json['data'][0]['id']
    return(cam_locations, subject_id)



'''Attach Image ER

This module defines a function called attach_image that attaches the classified
image with a bounding box, named according to the classified animal to an event
in earthranger using the api.
'''
import requests

def attach_image(event_id, img_byte, token, authorization, label):
    '''Attach Image

    This function attaches a classified and labeled image to an event in Earthranger
    as an alert system.

    Args:
    event_id: the unique id provided by Earthranger to be able to associate
        an image here with an event alert previously created for for each
        image to be added
    img_byte: the image to be posted in binary because that is how ER accepts it
    token: unique token for ER to authenticate http request, defined in config yml
    authorization: the other auth token for ER as defined in config yml, this was
        retrieved from the interactive api on ER
        https://<YOUR INSTANCE>.pamdas.org/api/v1.0/docs/interactive/
    label: the name of the animal identified to be sent, used as title of jpeg

    Returns: the http request response code to tell us if the call worked or not
    '''

    url = 'https://sagebrush.pamdas.org/api/v1.0/activity/event/' + event_id + '/files'

    unique_image_name = label + ".jpeg"

    content_type = 'application/jpeg'
    headers = {
        'X-CSRFToken': token,
        'Authorization': authorization
    }

    files = {'filecontent.file': (unique_image_name, img_byte, content_type)}
    response = requests.post(url, headers=headers, files=files)

    return response



'''Post Animal of Interest Log

This module defines a function called is_target which adds an observation
to a specific camera in Earthranger with the time and the fact that an animal of
interest was detected.
'''
from datetime import datetime
import requests
from earthranger_utils.get_cam_location import cam_location


def is_target(cam_name, token, authorization, label):
    '''Target animal historical log

    This function takes in the camera name and http api tokens only if
    an animal of interest was detected, and it then creates an observation for the specific
    camera it was detected at and logs the time so that there is a historical
    backlog for each camera of all its target animal detections.

    Args:
    cam_name: a string of the specific name of the camera that the image came from
        as it also is in Earthranger
    token: unique token for ER to authenticate http request, defined in config yml
    authorization: the other auth token for ER as defined in config yml, this was
        retrieved from the interactive api on ER
        https://<YOUR INSTANCE>.pamdas.org/api/v1.0/docs/interactive/
    '''
    
    headers = {
        'X-CSRFToken': token,
        'Authorization': authorization
        }

    current_time = datetime.utcnow()
    formatted_time = current_time.strftime('%Y-%m-%dT%H:%M:%S.%f') + 'Z'

    cam, subject_id = cam_location(cam_name, token, authorization)
    lat = cam[1]
    longi = cam[0]
    url = 'https://sagebrush.pamdas.org/api/v1.0/subject/' + subject_id + '/sources/'
    response = requests.get(url, headers=headers)
    response_json = response.json()

    source_id = response_json['data'][0]['id']

    url2 = 'https://sagebrush.pamdas.org/api/v1.0/observations/'

    payload = {"location": {"longitude": longi, "latitude": lat}, "recorded_at": formatted_time, "source": source_id, "device_status_properties": [{"value": label, "label": "animal", "units": ""}], "additional": {"animal": label}}

    requests.post(url2, headers=headers, json=payload)



from earthranger_utils.attach_image_er import attach_image
from earthranger_utils.get_cam_location import cam_location
from earthranger_utils.post_cougar_log import is_target
from earthranger_utils.post_event_er import post_event
